<?php
namespace cloud\apps\netoff;


function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';

	if(!is_supadmin($user->username)) {
		$where_str .= "`user` = ? and ";
		array_push($values, $user->username);
	}

	if(format_and_push($data, '_token_', $optional, '', 'int', false)) {
		$where_str .= "`token` = ? and ";
		array_push($values, $optional['_token_']);
	}

	if(format_and_push($data, 'unionid', $optional, '', 'string', false)) {
		$where_str .= "`unionid` = ? and ";
		array_push($values, $optional['unionid']);
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`unionid` like ? or `nick` like ? or `openid` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$order_map = array(
		"openid",
		"type",
		"unionid",
		"enable",
		"img",
		"nick",
		"session_time",
		"last_ip",
		"lastlogin",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}
	
	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `nick` asc';

	$sql = "select count(*) as `total` from cloud_platform.api_session_user $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$result = array (
			'total' => $rows[0]->total,
		);
	}
	else {
		return array (
			'total' => 0,
			'rows'	=> array()
		);
	}

	$sql = "select * from cloud_platform.api_session_user $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;
	$result['now'] = time();

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function enable($data)
{
	global $nidb, $user;


	$optional = array();
	if(is_supadmin($user->username)) {
		format_and_push($data, 'token', $optional, '', 'string', false);
		format_and_push($data, 'user', $optional, '', 'string', false);
	}
	else {
		$optional['user'] = $user->username;
		if(format_and_push($data, 'token', $optional, '', 'string', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定TOKEN。');
			return false;
		}
	}

	if(format_and_push($data, 'enable', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定操作。');
		return false;
	}
	
	$nick = array();
	if(!isset($optional['token']) || $optional['token'] == '') {
		if(!isset($optional['user']) || $optional['user'] == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定TOKEN 2。');
			return false;
		}

		$sql = "select `token`, `user`, `openid`, `nick` from cloud_platform.api_session_user where `user` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['user'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}
	else {
		if(preg_match("/^([a-zA-Z0-9-_,]{20,})$/", $optional['token'], $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "TOKEN“{$optional['token']}”不正确！");
			return false;
		}
		$ts = explode(',', $optional['token']);
		$tokens = array();
		foreach($ts as $val) {
			$t = trim($val);
			if($t != '')
				array_push($tokens, $t);
		}
		
		if(count($tokens) == 0) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "TOKEN“{$optional['token']}”不正确！");
			return false;
		}

		$str = "?";
		if(count($tokens) > 1)
			$str.= str_repeat(',?', count($tokens) - 1);

		$sql = "select `token`, `user`, `openid`, `nick` from cloud_platform.api_session_user where `token` in ({$str})";
		try {
			$sth = $nidb->prepare($sql);
			$sth->execute($tokens);
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}

	if(!is_array($uinfos) || count($uinfos) <= 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无此会话！");
		return false;
	}
	$tokens = array($optional['enable']);
	foreach($uinfos as $val)
		array_push($tokens, $val['token']);

	try {
		$str = "?";
		if(count($tokens) > 2)
			$str.= str_repeat(',?', count($tokens) - 1);

		$sql = "update cloud_platform.api_session_user set `enable` = ? where `token` in ({$str})";

		$sth = $nidb->prepare($sql);
		if($sth->execute($tokens)) {
			foreach($tokens as $key => $val) {
				cloud_insertlog($user->username, ($optional['enable']?'启用':'禁用') . "一键断网会话：用户={$uinfos[$key]['user']}，昵称={$uinfos[$key]['nick']}，OPENID={$val}");
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function save($data)
{
	global $nidb, $user;


	$where = array();
	$optional = array();
	if(format_and_push($data, 'token', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定TOKEN。');
		return false;
	}

	if(is_supadmin($user->username))
		format_and_push($data, 'enable', $optional, '', 'int', false);
	else
		$where['user'] = $user->username;

	format_and_push($data, 'nick', $optional, '', 'string', false);
	if(isset($data['img']))
		$optional['img'] = trim($data['img']);
	
	if(count($optional) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '参数不能为空。');
		return false;
	}

	if(isset($where['user']) && $where['user'] != '')
		$sql = "select * from cloud_platform.api_session_user where `token` = ? and `user` = ?";
	else
		$sql = "select * from cloud_platform.api_session_user where `token` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['token'], \PDO::PARAM_STR);
		if(isset($where['user']) && $where['user'] != '')
			$sth->bindParam(2, $where['user'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "会话用户“{$where['user']}”不存在！");
		return false;
	}

	try {
		if(update_data('cloud_platform`.`api_session_user', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$nick = $uinfo['nick'];
	$log = array();
	if(isset($optional['nick']) && $optional['nick'] != $nick) {
		array_push($log, "昵称");
		$nick = $optional['nick'];
	}
	if(isset($optional['img']) && $optional['img'] != $uinfo['img'])
		array_push($log, "头像");

	if(count($log)) {
		if($user->username != $uinfo['user'])
			$log = "修改一键断网会话用户：用户={$uinfo['user']}，昵称={$nick}，OPENID={$uinfo['openid']}，修改内容=" . implode(',', $log);
		else
			$log = "修改一键断网会话用户：昵称={$nick}，OPENID={$uinfo['openid']}，修改内容=" . implode(',', $log);

		cloud_insertlog($user->username, $log);
	}

	return $optional;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(is_supadmin($user->username)) {
		format_and_push($data, 'token', $optional, '', 'string', false);
		format_and_push($data, 'user', $optional, '', 'string', false);
	}
	else {
		$optional['user'] = $user->username;
		if(format_and_push($data, 'token', $optional, '', 'string', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定TOKEN。');
			return false;
		}
	}
	
	$nick = array();
	if(!isset($optional['token']) || $optional['token'] == '') {
		if(!isset($optional['user']) || $optional['user'] == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定TOKEN 2。');
			return false;
		}

		$sql = "select `token`, `user`, `openid`, `nick` from cloud_platform.api_session_user where `user` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['user'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}
	else {
		if(preg_match("/^([a-zA-Z0-9-_,]{20,})$/", $optional['token'], $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "TOKEN“{$optional['token']}”不正确！");
			return false;
		}
		$ts = explode(',', $optional['token']);
		$tokens = array();
		foreach($ts as $val) {
			$t = trim($val);
			if($t != '')
				array_push($tokens, $t);
		}
		
		if(count($tokens) == 0) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "TOKEN“{$optional['token']}”不正确！");
			return false;
		}

		$str = "?";
		if(count($tokens) > 1)
			$str.= str_repeat(',?', count($tokens) - 1);

		$sql = "select `token`, `user`, `openid`, `nick` from cloud_platform.api_session_user where `token` in ({$str})";
		try {
			$sth = $nidb->prepare($sql);
			$sth->execute($tokens);
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}

	if(!is_array($uinfos) || count($uinfos) <= 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无此会话！");
		return false;
	}
	$tokens = array();
	foreach($uinfos as $val)
		array_push($tokens, $val['token']);

	try {
		$str = "?";
		if(count($tokens) > 1)
			$str.= str_repeat(',?', count($tokens) - 1);

		$sql = "delete from cloud_platform.api_session_user where `token` in ({$str})";
		$sth = $nidb->prepare($sql);
		if($sth->execute($tokens)) {
			foreach($tokens as $key => $val) {
				cloud_insertlog($user->username, "删除一键断网会话：用户={$uinfos[$key]['user']}，昵称={$uinfos[$key]['nick']}，OPENID={$val}");
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}
