<?php
namespace cloud\apps\wechatinfo;


function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '`user` != `openid` and `enable` != 0 and ';
	if(format_and_push($data, 'openid', $optional, '', 'string', false)) {
		$where_str .= "`openid` = ? and ";
		array_push($values, $optional['openid']);
	}

	if(is_supadmin($user->username)) {
		if(format_and_push($data, 'user', $optional, '', 'string', false)) {
			$where_str .= "`user` = ? and ";
			array_push($values, $optional['user']);
		}
	}
	else {
		$where_str .= "`user` = ? and ";
		array_push($values, $user->username);
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`user` like ? or `nick` like ? or `openid` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$order_map = array(
		"openid",
		"unionid",
		"user",
		"enable",
		"discon",
		"expire",
		"img",
		"nick",
		"expire_time",
		"bind_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}
	
	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `nick` asc';

	$sql = "select count(*) as `total` from cloud_platform.weixin_user $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		return array(
			'total' => 0,
			'rows'	=> array()
		);
	}

	$sql = "select * from cloud_platform.weixin_user $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;
	$result['now'] = time();

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function info($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'openid', $optional, '', 'string', false) == false) {
		if(format_and_push($data, 'create', $optional, '', 'int', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, 'openid不能为空。');
			return false;
		}
	}

	if(is_supadmin($user->username))
		format_and_push($data, 'user', $optional, '', 'string', false);
	else
		$optional['user'] = $user->username;

	try {
		$sql = "select * from cloud_platform.weixin_user";
		if(isset($optional['openid']) && $optional['openid'] != '')
			$sql.= " where `openid` = ?";
		else
			$sql.= " where `create_time` = ?";

		if(isset($optional['user']) == false || $optional['user'] == '')
			$sql.= " and `user` = ?";

		$sth = $nidb->prepare($sql);
		if(isset($optional['openid']) && $optional['openid'] != '')
			$sth->bindParam(1, $optional['openid'], \PDO::PARAM_STR);
		else
			$sth->bindParam(1, $optional['create'], \PDO::PARAM_INT);

		$sth->bindParam(2, $optional['user'], \PDO::PARAM_STR);

		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0)
		return $rows[0];

	return false;
}

function add($data)
{
	global $nidb, $user;


	$optional = array();
	if(is_supadmin($user->username)) {
		if(format_and_push($data, 'user', $optional, '', 'string', false) == false) {
			$optional['user'] = $user->username;
		}
	}
	else
		$optional['user'] = $user->username;

	if(format_and_push($data, 'openid', $optional, '', 'string', false) == false) {
		if($optional['user'] == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID。');
			return false;
		}
		$optional['openid'] = $optional['user'];
	}
	format_and_push($data, 'img', $optional, '', 'string', false);
	format_and_push($data, 'nick', $optional, '', 'string', false);
	format_and_push($data, 'enable', $optional, '', 'int', false);
	format_and_push($data, 'discon', $optional, '', 'int', false);
	format_and_push($data, 'expire', $optional, '', 'int', false);

	$sql = "select `username` from `cloud_users` where `username` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['user'], \PDO::PARAM_STR);
		$sth->execute();
		$cuinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$cuinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "“{$optional['user']}”帐号不存在！");
		return false;
	}
	
	$sql = "select `openid` from cloud_platform.weixin_user where `openid` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['openid'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "此帐号“{$uinfo['user']}”已创建！");
		return false;
	}

	$optional['create_time'] = time();

	try {
		if(insert_data('cloud_platform`.`weixin_user', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $optional;
}

function rebind($data)
{
	global $nidb, $user;

	$where = array();
	$optional = array();
	if(!is_supadmin($user->username))
		$where['user'] = $user->username;
//	else 
//		format_and_push($data, 'user', $where, '', 'string', false);

	if(format_and_push($data, 'openid', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID。');
		return false;
	}

	if(isset($where['user']) && $where['user'] != '')
		$sql = "select `openid`, `nick`, `user`, `create_time` from cloud_platform.weixin_user where `openid` = ? and `user` = ?";
	else
		$sql = "select `openid`, `nick`, `user`, `create_time` from cloud_platform.weixin_user where `openid` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['openid'], \PDO::PARAM_STR);
		if(isset($where['user']) && $where['user'] != '')
			$sth->bindParam(2, $where['user'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "关联帐号“{$where['user']}”不存在！");
		return false;
	}

	$optional['expire_time'] = time() + 120;

	try {
		if(update_data('cloud_platform`.`weixin_user', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// cloud_insertlog($user->username, "关联微信：用户={$where['user']}，昵称={$nick}");

	$optional['create_time'] = $uinfo['create_time'];
	$optional['user'] = $uinfo['user'];
	$optional['openid'] = $uinfo['openid'];

	return $optional;
}

function qrcode_rebind($data)
{
	global $nidb, $user;

	$where = array();
	$optional = array();
	if(format_and_push($data, 'openid', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID。');
		return false;
	}

	if(is_supadmin($user->username))
		format_and_push($data, 'user', $where, '', 'string', false);
	else
		$where['user'] = $user->username;

	$guid = _get_uuidgen();
	if (!$guid) return false;

	$sql = "select `expire_time`, `user` from cloud_platform.weixin_user where `openid` = ?";
	if(isset($where['user']))
		$sql.= " and `user` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['openid'], \PDO::PARAM_STR);
		if(isset($where['user']))
			$sth->bindParam(2, $where['user'], \PDO::PARAM_STR);
		if($sth->execute())
			$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
		else
			return false;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无此微信帐号“{$where['openid']}”！");
		return false;
	}

	$appid = "wxc51276c9c38e2344";
	$redirect_uri = "https://weixin.panabit.com/panacloud/wxbind.php";

	$state = "cloud2 {$guid} {$uinfo['user']} {$uinfo['expire_time']}";
	// file_put_contents('/tmp/wxqrcode.log', $state);
	// 二维码内容 
	$value = "https://open.weixin.qq.com/connect/oauth2/authorize?appid={$appid}&redirect_uri={$redirect_uri}&response_type=code&scope=snsapi_userinfo&state={$state}#wechat_redirect";

	// 容错级别
	$errorCorrectionLevel = 'L';

	// 生成图片大小
	$matrixPointSize = 5;

	// 生成二维码图片 
	$path = dirname(__DIR__) . '/assets/qrcode';
	if(is_dir($path) == false) {
		if(mkdir($path, 755) === false) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '创建二维码图片文件夹失败！' . $path);
			return false;
		}
	}
	$QRfile = tempnam($path, '');
	if(!$QRfile) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '创建二维码图片文件失败！' . $path);
		return false;
	}

	\QRcode::png($value, $QRfile, $errorCorrectionLevel, $matrixPointSize, 2); 

	$QR = imagecreatefromstring(file_get_contents($QRfile)); 

	//二维码图片宽度 
	$QR_width = imagesx($QR);

	//二维码图片高度 
	$QR_height = imagesy($QR);
	$from_width = ($QR_width - 11) / 2; 

	//imagecopyresampled($QR, "", $from_width, $from_width, 0, 0, $logo_qr_width, $logo_qr_height, $logo_width, $logo_height); 
	header("Content-type: image/png");
	ImagePng($QR);
	unlink($QRfile);

	return true;
}

function save($data, $create_time = 0)
{
	global $nidb, $user;


	$where = array();
	$optional = array();
	if(format_and_push($data, 'openid', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID。');
		return false;
	}

	if(is_supadmin($user->username))
		format_and_push($data, 'enable', $optional, '', 'int', false);
	else
		$where['user'] = $user->username;


	format_and_push($data, 'discon', $optional, '', 'int', false);
	format_and_push($data, 'expire', $optional, '', 'int', false);
	format_and_push($data, 'nick', $optional, '', 'string', false);
	if(isset($data['img']))
		$optional['img'] = trim($data['img']);
	
	if(count($optional) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '参数不能为空。');
		return false;
	}

	if(isset($where['user']) && $where['user'] != '')
		$sql = "select * from cloud_platform.weixin_user where `openid` = ? and `user` = ?";
	else
		$sql = "select * from cloud_platform.weixin_user where `openid` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['openid'], \PDO::PARAM_STR);
		if(isset($where['user']) && $where['user'] != '')
			$sth->bindParam(2, $where['user'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "关联帐号“{$where['user']}”不存在！");
		return false;
	}

	if($create_time)
		$optional['create_time'] = $create_time;
	
	try {
		if(update_data('cloud_platform`.`weixin_user', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$nick = $uinfo['nick'];
	$log = array();
	if(isset($optional['nick']) && $optional['nick'] != $nick) {
		array_push($log, "微信昵称");
		$nick = $optional['nick'];
	}
	if(isset($optional['img']) && $optional['img'] != $uinfo['img'])
		array_push($log, "微信头像");
	if(isset($optional['discon']) && $optional['discon'] != $uinfo['discon'])
		array_push($log, "离线通知" . ($optional['discon']?'打开':'关闭'));
	if(isset($optional['expire']) && $optional['expire'] != $uinfo['expire'])
		array_push($log, "到期通知" . ($optional['expire']?'打开':'关闭'));

	if(count($log)) {
		if($user->username != $uinfo['user'])
			$log = "修改关联微信配置：用户={$uinfo['user']}，昵称={$nick}，OPENID={$uinfo['openid']}，修改内容=" . implode(',', $log);
		else
			$log = "修改关联微信配置：昵称={$nick}，OPENID={$uinfo['openid']}，修改内容=" . implode(',', $log);

		cloud_insertlog($user->username, $log);
	}

	return $optional;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(is_supadmin($user->username)) {
		format_and_push($data, 'openid', $optional, '', 'string', false);
//		format_and_push($data, 'user', $optional, '', 'string', false);
	}
	else {
		$optional['user'] = $user->username;

		if(format_and_push($data, 'openid', $optional, '', 'string', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID。');
			return false;
		}
	}
	
	$openids = array();
	$nick = array();
	if(!isset($optional['openid']) || $optional['openid'] == '') {
		if(!isset($optional['user']) || $optional['user'] == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定OPENID 2。');
			return false;
		}

		$sql = "select `openid`, `nick` from cloud_platform.weixin_user where `user` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['user'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}
	else {
		$sql = "select `user`, `openid`, `nick` from cloud_platform.weixin_user where `openid` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['openid'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfos = $sth->fetchAll(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}

	if(!is_array($uinfos) || count($uinfos) <= 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无此帐号！");
		return false;
	}
	foreach($uinfos as $val)
		array_push($openids, $val['openid']);

	try {
		$str = "?";
		if(count($openids) > 1)
			$str.= str_repeat(',?', count($openids) - 1);

		$sql = "update cloud_platform.weixin_user set `enable` = 0 where `openid` in ({$str})";
		$sth = $nidb->prepare($sql);
		if($sth->execute($openids)) {
			foreach($openids as $key => $val) {
				cloud_insertlog($user->username, "删除微信通知关联：用户={$uinfos[$key]['user']}，昵称={$uinfos[$key]['nick']}，OPENID={$val}");
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function getconfig($tag, $def)
{
	$conf = LOG_CONF;

	if(!isset($def))
		$def = 0;
	$v = '';
	if (file_exists($conf)) {
		$lines = file($conf);
		foreach($lines as $val) {
			if (strstr($val, $tag) !== false) {
				$ds = explode('=', $val);
				$v = trim($ds[1], "\r\n");
				return $v;
			}
		}
	}
	
	return $def;
}

function setconfig($tag, $v)
{
	$conf = LOG_CONF;
	
	$tag_exists = 0;
	
	if (file_exists($conf)) {
		$lines = file($conf);
		
		if (($fp = fopen($conf, "w")) == false)
			return false;
		if (flock($fp, LOCK_EX)) {	
			foreach($lines as $val) {
				if (strstr($val, $tag."=") !== false) {
					fwrite($fp, "$tag=$v\n");
					$tag_exists = 1;
				}else
					fwrite($fp, $val);
				fflush($fp);
			}
		
			if ($tag_exists == 0) {
				fwrite($fp, "$tag=$v\n");
				fflush($fp);
			}
			flock($fp, LOCK_UN);
		}
		fclose($fp);

		return true;
	}

	return false;
}

function msgname($data)
{
	global $user;

	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
		return false;
	}

	if(!isset($data['name']))  {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定名称！');
		return false;
	}

	$name = trim($data['name']);

	$wxname = trim(getconfig("cloud_weixin_name", ''));
	if(!setconfig("cloud_weixin_name", $name)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, "“{$name}”保存失败！");
		return false;
	}

	cloud_insertlog($user->username, "修改微信通知标题：{$wxname} -> {$name}");

	set_errmsg(MSG_LEVEL_ARG, __function__, "修改成功！");
	return true;
}

function _get_uuidgen()
{
	$ret = '';

	if (file_exists("/etc/uuidgen.conf")) {
		$ret = trim(file_get_contents("/etc/uuidgen.conf"), " \n\r\t\v\x00");
		if(empty($ret) == false)
			return $ret;
	}
	else {
		exec('/bin/uuidgen >> /etc/uuidgen.conf', $out, $ret);
		if($ret)
			set_errmsg(MSG_LEVEL_DEF, __function__, "UUIDGEN.CONF_NEXISTS");
		else {
			$ret = trim(file_get_contents("/etc/uuidgen.conf"), " \n\r\t\v\x00");
			if(empty($ret) == false)
				return $ret;
		}
	}

	return false;
}

function weixin_config($data)
{
	global $nidb, $user;

	$result = array(
		'discon'	=> 0,
		'expire'	=> 0,
		'wxname'	=> '',
	);

	$objs = getconfig("cloud_weixin_relation_obj", '');
	if($objs) {
		$ds = explode(',', $objs);
		foreach($ds as $val) {
			if ($val == "discon")
				$result['discon'] = 1;
			if ($val == "expire_license")
				$result['expire'] = 1;
		}
	}

	$result['wxname'] = trim(getconfig("cloud_weixin_name", ''));

	return $result;
}

function weixin_unbind($data)
{
	global $user;
	
	$optional = array();

	if(format_and_push($data, 'openid', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '操作对象不能为空。');
		return false;
	}
	if(preg_match("/^([a-zA-Z0-9-_]{20,})$/", $optional['openid'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, "用户“{$optional['openid']}”不正确！");
		return false;
	}

	$guid = _get_uuidgen();
	if ($guid) {
//		$ret = httpPost("http://www.panabit.com/wxcloud/user_unbind.php?guid={$guid}&openid={$optional['openid']}", array(), false);
		$ret = httpPost("http://www.panabit.com/wxcloud/user_unbind1.php?guid={$guid}&openid={$optional['openid']}&user={$user->username}", array(), false);
		$data = json_decode($ret, true);
		if(json_last_error() == JSON_ERROR_NONE) {
			if($data['ret'] != 0) {
				set_errmsg(MSG_LEVEL_DEF, __function__, $data['out']);
				return false;
			}
		}
	}

	return true;
}

function weixin_msgswitch($data)
{
	$optional = array();

	$discon = 0;
	if(isset($data['discon']) && ($discon = intval($data['discon'])) != 0) {
		$discon = 1;
		array_push($optional, 'discon');
	}

	if(isset($data['expire']) && intval($data['expire']) != 0) {
		array_push($optional, 'expire_license');
	}

	$cmd = DATAEYE . " device config notify_discon={$discon}";
	exec($cmd, $out, $ret);

	setconfig("cloud_weixin_relation_obj", implode(',', $optional));

	return true;
}
